import pandas as pd


def feature_importance(X, model):
    feat_imp = pd.DataFrame({"columns": X.columns, "feature_importance": model.feature_importances_})
    return feat_imp


def drop_irrelevant_features(X, feat_imp, thresh=1e-5):
    cols = feat_imp[feat_imp.feature_importance < thresh].columns
    return X.drop(columns=cols)



from hyperopt import STATUS_OK, Trials, fmin, hp, tpe
from src.ds_prod.preprocessing import validation_split
from sklearn.metrics import mean_squared_error
from xgboost import XGBRegressor
import numpy as np


class HOpt:
    def __init__(self, x, y, val_split=0.8):
        [tr, te] = validation_split(x, val_split)
        self.X_train = x.iloc[tr]
        self.X_val = x.iloc[te]
        self.Y_train = y.iloc[tr]
        self.Y_val = y.iloc[te]
        self.space = {
            'n_estimators': hp.randint('n_estimators', 50, 200),
            'learning_rate': hp.choice('learning_rate', np.arange(0.01, 0.1, 0.01)),
            'max_depth': hp.choice('max_depth', np.arange(5, 8, 1, dtype=int)),
            'min_child_weight': hp.choice('min_child_weight', np.arange(0, 8, 1, dtype=int))
        }

    def get_space(self):
        return self.space

    def set_space(self, newspace):
        self.space = newspace

    def f(self, params):
        xgb1 = XGBRegressor(tree_method='gpu_hist', **params)
        xgb1.fit(self.X_train, self.Y_train)
        y_pred = xgb1.predict(self.X_val)
        acc = mean_squared_error(self.Y_val, y_pred)
        return {'loss': acc, 'status': STATUS_OK}

    def optimize(self, evals=50):
        trials = Trials()
        best = fmin(fn=self.f, space=self.space, algo=tpe.suggest,
                    trials=trials,
                    max_evals=evals)
        return [best, trials]



from itertools import product, chain
from sklearn.model_selection import TimeSeriesSplit


class CustomValidationFolds:
    """Creates validation class that splits using moving window for the ordered dates of the data"""

    def __init__(self, **kwargs):
        self.tscv = TimeSeriesSplit(**kwargs)

    def get_n_splits(self,X=None, y=None, groups=None):
        """gets number of splits

        Yields
        ------
        n_splits: int
            number of splits
        """
        return self.tscv.get_n_splits()

    def split(self, X, y=None, groups=None):
        """ splits data X into n_splits  by time range.

        Parameters
        ----------
        X : array-like of shape (n_samples, n_features)
            Training data, where `n_samples` is the number of samples
            and `n_features` is the number of features.
        y : array-like of shape (n_samples,)
            Always ignored, exists for compatibility.
        groups : array-like of shape (n_samples,)
            Always ignored, exists for compatibility.
        Yields
        ------
        list of [train, test] pairs
        train : list
            The training set indices for that split.
        test : list
            The testing set indices for that split.
        """
        months = set(X.month)
        years = set(X.year)
        ym_list = list(product(years, months))[:-2]
        inds = list(self.tscv.split(ym_list))
        train_test_split = []
        train = inds[0][0]
        tr = X[X.apply(lambda x: (x[0], x[1]) in ym_list[min(train):max(train) + 1], axis=1)].index
        for i in range(0, self.get_n_splits()):
            ind = inds[i][1]
            te = X[X.apply(lambda x: (x[0], x[1]) in ym_list[min(ind):max(ind) + 1], axis=1)].index
            train_test_split.append((tr, te))
            tr = chain(tr,te)
        return train_test_split


def validation_split(X, train_ratio=0.8):
    """
    splits train X into train and validation by splitting the timeseries by the ratio

    The function takes the ordered year-months pairs from the data set, divides them
    and the dataset is divided into

    Parameters
    -------
    X: DataFrame with month and year columns
        the dataset to be divided
    train_ratio: float
        the ratio of time to go into train

    Yields
    -------
    train : list
            The training set indices for that split.
    test : list
            The testing set indices for that split.
    """
    months = set(X.month)
    years = set(X.year)
    ym_list = list(product(years, months))[:-2]
    train_num = int(train_ratio * len(ym_list))
    print(train_num)
    ind = X.apply(lambda x: (x[0], x[1]) in ym_list[:train_num], axis=1)
    train = X[ind].index
    test = X.drop(train).index
    return [train, test]



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="ds-trainee-prod",                     # This is the name of the package
    version="0.0.11",                        # The initial release version
    author="Natallia Zhamaitsiak",                     # Full name of the author
    description="My ds trainee project",
    long_description=long_description,      # Long description read from the the readme file
    long_description_content_type="text/markdown",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],                                      # Information to filter the project on PyPi website
    python_requires='>=3.6',  
    py_modules=["ds_trainee_prod"],
    package_dir={"": "src"},
    packages=['ds_prod'],
    include_package_data=True,
    install_requires=[
        'scikit-learn',
        'xgboost',
        'numpy',
        'pandas',
        'hyperopt',
      ],
                   # Install other dependencies if any
)
